/*
  SilentStepStick TMC5160 Example
  Rsense: 0.075 Ohm

  TMCStepper library required:
  https://github.com/teemuatlut/TMCStepper

  Example source code free to use.
  Further information: https://learn.watterott.com/license/
  Modified for BeefyStepper
  7/25/22    -jkl
*/

#include <TMCStepper.h>
#include <AccelStepper.h>
#include <elapsedMillis.h>

// Note: You also have to connect GND, 5V/VIO and VM.
//       A connection diagram can be found in the schematics.
const byte stopButton = 5;
const byte signalLED = 10;

#define EN_PIN           7 // Enable
#define DIR_PIN          16 // Direction
#define STEP_PIN         17 // Step
#define CS_PIN           8 // Chip select


#define R_SENSE 0.05f // Match to your driver
// SilentStepStick series use 0.11
// UltiMachine Einsy and Archim2 boards use 0.2
// Panucatt BSD2660 uses 0.1
// Watterott TMC5160 uses 0.075
// BeefyStepper uses 0.05


TMC5160Stepper tmc = TMC5160Stepper(CS_PIN, R_SENSE); //use hardware SPI
AccelStepper myStepper(AccelStepper::DRIVER, STEP_PIN, DIR_PIN);
// State definitions
#define RUN 01
#define STOPPED 02
// State variable
int state;

elapsedMillis printTime;

void setup()
{
  //set pins
  pinMode(EN_PIN, OUTPUT);
  digitalWrite(EN_PIN, HIGH); //deactivate driver (LOW active)
  pinMode(stopButton, INPUT_PULLUP);
  pinMode(signalLED, OUTPUT);

  pinMode(CS_PIN, OUTPUT);
  digitalWrite(CS_PIN, HIGH);

  // These for run()
  myStepper.setMaxSpeed(10000.);   // this is the target speed for run()
  myStepper.setAcceleration(1000.);
  myStepper.moveTo(40000);


  //init serial port
  Serial.begin(115200); //init serial port and set baudrate
  while (!Serial); //wait for serial port to connect
  Serial.println("\nStart...");
  SPI.begin();          // This is required! Nothing happens unless this is done.
  //set driver config
  tmc.begin();
  tmc.toff(4); //off time
  tmc.blank_time(24); //blank time
  //tmc.en_pwm_mode(1); //enable extremely quiet stepping
  tmc.microsteps(4); // microsteps
  Serial.print ("GSTAT: ");
  Serial.println(tmc.GSTAT());
  tmc.GSTAT(0x07);
  Serial.println(tmc.GSTAT());
  tmc.rms_current(5000); // RMS current in mA
  tmc.drvstrength(0);
  tmc.filt_isense(0);   // values of 0 to 3 are possible
  //Serial.println (tmc.drvstrength());
  digitalWrite(signalLED, HIGH);
  //outputs on (LOW active)
  digitalWrite(EN_PIN, LOW);
  state = RUN;
}

void loop()
{
  if (digitalRead(stopButton) == LOW) {     // makes motor stop or resume
    if ( state == RUN) {          // Time to stop!
      state = STOPPED;

      myStepper.stop();
      myStepper.runToPosition();   // go immediately to stop position!
      digitalWrite(signalLED, LOW);
      digitalWrite(EN_PIN, HIGH);
      // while (true);           // just hang
      Serial.print(myStepper.speed());
      Serial.print("  ");
      Serial.println(myStepper.currentPosition());
      //Serial.print("  ");
      //Serial.println(tmc.rms_current());
      delay(1000); //debounce time - may want to require release?
      printTime = 0;     // reset timer
    } else {          // Time to Run!
      Serial.print(myStepper.speed());
      Serial.print("  ");
      Serial.println(myStepper.currentPosition());
      //Serial.print("  ");
      //Serial.println(tmc.rms_current());
      digitalWrite(signalLED, HIGH);
      digitalWrite(EN_PIN, LOW);
      myStepper.moveTo(40000);        // reset the target
      state = RUN;
      delay(1000); //debounce time - may want to require release?
      printTime = 0;     // reset timer
    }
  }
  // int lCount = 0; // elapsed seconds

  switch (state) {
    case RUN:
  /*    if (printTime >= 1000) {
        printTime = 0;
        //    lCount++;
        Serial.print(myStepper.speed());
        Serial.print("  ");
        Serial.println(myStepper.currentPosition());
        //Serial.print("  ");
        //Serial.println(tmc.rms_current());
        if (tmc.diag0_error()) {
          Serial.println(F("DIAG0 error"));
        }
        if (tmc.ot())         {
          Serial.println(F("Overtemp."));
        }
        if (tmc.otpw())       {
          Serial.println(F("Overtemp. PW"));
        }
        if (tmc.s2ga())       {
          Serial.println(F("Short to Gnd A"));
        }
        if (tmc.s2gb())       {
          Serial.println(F("Short to Gnd B"));
        }
        if (tmc.ola())        {
          Serial.println(F("Open Load A"));
        }
        if (tmc.olb())        {
          Serial.println(F("Open Load B"));
        }
      }*/

      // If at the end of travel go to the other end
      if (myStepper.distanceToGo() == 0)
        myStepper.moveTo(-myStepper.currentPosition());

      myStepper.run();
      break;
    case STOPPED:     // This makes sure nothing happens if stopped
      break;
    default:
      Serial.println("Unknown case!");
  }
}
